]> git.plutz.net Git - flarejs/blobdiff - engine.js
process INCLUDE directive when loading file
[flarejs] / engine.js
index f3481c713f96d1c96f1162d79bc01bc0b213cd5b..0390d83372bb31f1000026bda84813bc6e5a68b9 100644 (file)
--- a/engine.js
+++ b/engine.js
@@ -26,8 +26,23 @@ function Textfile(txtfile) {
   fetch.open("GET", txtfile, false);
   fetch.send();
 
+  function merge(obj, file) {
+    for (each in file) {
+      if (Array.isArray(obj[each]) && Array.isArray(file[each])) obj[each] = [ ...obj[each], ...file[each] ];
+      else if (Array.isArray(obj[each])) obj[each] = [ ...obj[each], file[each] ];
+      else if (Array.isArray(file[each])) obj[each] = [ obj[each], ...file[each] ];
+      else if (each in obj) obj[each] = [ obj[each], file[each] ];
+      else obj[each] = file[each];
+    }
+  }
+
   var line, key, value, ref = this, section;
   for (line of fetch.responseText.split('\n')) switch(true) {
+    // INCLUDE file
+    case /^INCLUDE .+$/.test(line):
+      //console.log("Including: " + line.split(/ /)[1]);
+      merge(ref, new Textfile(line.split(/ /)[1]));
+      break;
     // section headers that always become array elements
     case /^\[(npc|event|layer|dialog)\]$/.test(line):
       section = line.split(/[\]\[]/)[1]; ref = {};
@@ -41,6 +56,10 @@ function Textfile(txtfile) {
       else if (!Array.isArray(this[section])) this[section] = [this[section],ref];
       else if ( Array.isArray(this[section])) this[section].push(ref);
       break;
+    // Clear section on empty line
+    case /^ *$/.test(line):
+      ref = this;
+      break;
     // frame definition from animation files
     // frame=#frame,direction,srcX,srcY,width,height,dstX,dstY
     case /^frame=[0-9]+,[0-7](,[0-9-]+){6}$/.test(line):
@@ -184,7 +203,7 @@ function Map(textdef) {
   }
 
   // draw the entire map, including all mobs
-  // mobs in an array of all heros, enemies, loot, npcs, etc.
+  // mobs is an array of all heros, enemies, loot, npcs, etc.
   this.draw = function(mobs) {
     const bg = this.info.layer.find(l => l.type == "background").data;
     const ob = this.info.layer.find(l => l.type == "object").data;
@@ -230,8 +249,8 @@ function Map(textdef) {
   }
 }
 
-function Mob(textdef) {
-  // any mobile object, mostly for gfx representation
+function Sprite(textdef) {
+  // sprite object, mostly for gfx representation
   // may be hero, enemy, loot, npc, etc.
   this.position = [0, 0];
   const info = gamedata.load(textdef);
@@ -252,14 +271,14 @@ function Mob(textdef) {
       info[animation].frame[i] = [[ i * rs[0], 0, rs[0], rs[1], ro[0], ro[1]]];
   }
 
// place mob on x/y pixel coordinates
   // e.g. when spawning, walking, etc.
   this.place = function(x, y) {
     this.position[0] = x, this.position[1] = y;
     return this;
   }
 
// change facing direction of mob
   this.direct = function(d) { direction = d % 8; return this; }
 
   // set animation cycle for drawing
@@ -274,7 +293,7 @@ function Mob(textdef) {
     return this;
   }
 
// draw this mob to screen position x/y
   this.draw = function(x, y){
     var f, a = info[animation];
     var f, frame = ( performance.now() - frametime ) * a.frames / a.duration | 0;
@@ -321,7 +340,7 @@ function Mob(textdef) {
 
 function Npc(textdef){
   this.info = gamedata.load(textdef);
-  const avatar = new Mob(this.info.gfx);
+  const avatar = new Sprite(this.info.gfx);
 
   this.place = function(x, y) {
     avatar.place(x,y);
@@ -338,39 +357,39 @@ function Hero(gender = "female", hair = "short"){
   // Object for player character
   // unique in single player
   this.position = [0,0];
-  this.stats = gamedata.load("/engine/stats.txt");
-  const layers = gamedata.load("/engine/hero_layers.txt");
+  this.stats = gamedata.load("engine/stats.txt");
+  const layers = gamedata.load("engine/hero_layers.txt");
   var direction = 0, animation = "stance";
   hair = (gender == "female")?"long":hair;
 
-  // hero consists of multiple mobs, one for each clothing/item overlay
+  // hero consists of multiple sprites, one for each clothing/item overlay
   var limbs = {
-    head : new Mob("/animations/avatar/"+gender+"/head_"+hair+".txt"),
-    chest: new Mob("/animations/avatar/"+gender+"/cloth_shirt.txt"),
-    hands: new Mob("/animations/avatar/"+gender+"/default_hands.txt"),
-    legs : new Mob("/animations/avatar/"+gender+"/cloth_pants.txt"),
-    feet : new Mob("/animations/avatar/"+gender+"/default_feet.txt"),
+    head : new Sprite("animations/avatar/"+gender+"/head_"+hair+".txt"),
+    chest: new Sprite("animations/avatar/"+gender+"/cloth_shirt.txt"),
+    hands: new Sprite("animations/avatar/"+gender+"/default_hands.txt"),
+    legs : new Sprite("animations/avatar/"+gender+"/cloth_pants.txt"),
+    feet : new Sprite("animations/avatar/"+gender+"/default_feet.txt"),
     main : null, // main hand, e.g. melee weapon, magic weapon
     off  : null  // off hand, e.g. shield, ranged weapon
   }
 
   // "dress" the player, i.e. assign clothing/item to limb
   this.dress = function(limb, item) {
-    limbs[limb] = new Mob("/animations/avatar/"+gender+"/"+item+".txt")
+    limbs[limb] = new Sprite("animations/avatar/"+gender+"/"+item+".txt")
     limbs[limb].place(this.position[0], this.position[1]).direct(direction);
     this.animate(animation);
     return this;
   }
 
// // Wrapper functions for Mob class // //
 
-  // place all mobs beloning to hero (when spawning, walking, teleporting, etc.)
+  // place all sprites beloning to hero (when spawning, walking, teleporting, etc.)
   this.place   = function(x,y) {
     this.position[0] = x, this.position[1] = y;
     for (var limb in limbs) limbs[limb] && limbs[limb].place(x,y);
     return this;
   }
-  // change facing direction of hero (i.e. of all mobs)
+  // change facing direction of hero (i.e. of all sprites)
   this.direct  = function(d)   {
     direction = d;
     for (var limb in limbs) limbs[limb] && limbs[limb].direct(d);
@@ -410,6 +429,8 @@ function Controls(hero){
   }
   var keys = [], click = [];
 
+  loadevents();
+
   window.addEventListener("keydown", e => keys[e.keyCode] = true );
   window.addEventListener("keyup"  , e => keys[e.keyCode] = false);
   window.addEventListener("click", m => click = [ map.mapX(m.clientX), map.mapY(m.clientY) ] );
@@ -446,36 +467,50 @@ function Controls(hero){
   // Process on_trigger events at pixel position x/y
   function triggers(x, y) {
     const i = map.tileAt(x,y);
-    var trigger, item;
 
-    if (map.triggers[i]) for ( trigger of map.triggers[i].filter(
-                               t => (!t.requires_status)     ? true : t.requires_status.find(s => s in qstatus)
-                     ).filter( t => (!t.requires_not_status) ? true : !t.requires_not_status.find(s => s in qstatus) )
-    ) {
+    if ( map.triggers[i] ) events (
+      map.triggers[i].filter( t => (!t.requires_status)     ? true : t.requires_status.find(s => qstatus[s])
+                    ).filter( t => (!t.requires_not_status) ? true : !t.requires_not_status.find(s => qstatus[s]) )
+    );
+  }
+
+  function loadevents() {
+    if ( map.info.event ) events(
+      map.info.event.filter( ev => ev.activate == "on_load"
+        ).filter( t => (!t.requires_status)     ? true : t.requires_status.find(s => qstatus[s])
+        ).filter( t => (!t.requires_not_status) ? true : !t.requires_not_status.find(s => qstatus[s]) )
+    )
+  }
+
+  function events( events ) {
+    var ev, item;
+
+    for ( ev of events ) {
       // game status modification
-      if ( trigger.set_status ) for (item of trigger.set_status) {
+      if ( ev.set_status ) for (item of ev.set_status) {
         qstatus[item] = true;
       }
-      if ( trigger.unset_status ) for (item of trigger.unset_status) {
+      if ( ev.unset_status ) for (item of ev.unset_status) {
         qstatus[item] = false;
       }
-      if ( item = trigger.msg ) {
+      if ( item = ev.msg ) {
         console.log(item);
       }
       // intramap (i.e. teleporters)
-      if ( item = trigger.intramap ){
+      if ( item = ev.intramap ){
         hero.place( map.xOf(item[0], item[1]), map.yOf(item[0], item[1]) );
         map.center( map.xOf(item[0], item[1]), map.yOf(item[0], item[1]) );
       }
       // mapmod (e.g. opening doors, activating platforms, changing terrain, ...)
-      if ( trigger.mapmod ) for (item of trigger.mapmod) {
+      if ( ev.mapmod ) for (item of ev.mapmod) {
           map.info.layer.find(l => l.type == item[0]).data[item[2] * map.info.header.width + item[1]] = item[3];
       }
       // intermap (i.e. enter new area)
-      if ( item = trigger.intermap ){
+      if ( item = ev.intermap ){
         map = new Map(item[0]);
         hero.place( map.xOf(item[1], item[2]), map.yOf(item[1], item[2]) );
         map.center( map.xOf(item[1], item[2]), map.yOf(item[1], item[2]) );
+        loadevents();
         break;  // further events would not be valid on new map
       }
     }
@@ -525,9 +560,9 @@ window.addEventListener("resize", function(){
   canvas.canvas.height = window.innerHeight;
 });
 
-// player = new Mob("animations/enemies/zombie.txt"); player.stats = { speed: 3 };
+// player = new Sprite("animations/enemies/zombie.txt"); player.stats = { speed: 3 };
 player = new Hero();
-map = new Map("/maps/perdition_harbor.txt");
+map = new Map("maps/perdition_harbor.txt");
 player.place(map.xOf(map.info.header.hero_pos[0], map.info.header.hero_pos[1]),
              map.yOf(map.info.header.hero_pos[0], map.info.header.hero_pos[1]));
 player.direct(6).stance();
@@ -252,14 +271,14 @@ function Mob(textdef) {
   }
 
-  // place mob on x/y pixel coordinates
+  // place sprite on x/y pixel coordinates
   // e.g. when spawning, walking, etc.
   this.place = function(x, y) {
@@ -252,14 +271,14 @@ function Mob(textdef) {
   }
 
-  // change facing direction of mob
+  // change facing direction of sprite
   this.direct = function(d) { direction = d % 8; return this; }
 
@@ -274,7 +293,7 @@ function Mob(textdef) {
   }
 
-  // draw this mob to screen position x/y
+  // draw this sprite to screen position x/y
   this.draw = function(x, y){
     var f, a = info[animation];
@@ -338,39 +357,39 @@ function Hero(gender = "female", hair = "short"){
   }
 
-  // // Wrapper functions for Mob class // //
+  // // Wrapper functions for Sprite class // //
 